Exception: Fractor::ClosedQueueError
- Inherits:
-
StandardError
- Object
- StandardError
- Fractor::ClosedQueueError
- Defined in:
- lib/fractor.rb
Overview
Exception raised when trying to push to a closed queue
Exception raised when trying to push to a closed queue